var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/* eslint-disable react/prop-types */
import { DataObject } from "@fluidframework/aqueduct";
import { SharedJson1 } from "@fluid-experimental/sharejs-json1";
import { SharedMap } from "@fluidframework/map";
import * as React from "react";
import * as ReactDOM from "react-dom";
import ReactJson from "react-json-view";
// import {produce} from "immer";
// import { Doc, type as Json1OTType, JSONOp, replaceOp, insertOp, moveOp, removeOp, Path } from "ot-json1";
// const myJson = {
//     a: "topLevel",
//     b: {
//         b1: "nested 1 level",
//         b2: {
//             b22: "nested 2 levels",
//         },
//     },
// };
// type ArItem = string | number | boolean | Doc;
// const myArray = {
//     array: [],
// };
export class ArrayDataObject extends DataObject {
    // [x: string]: any;
    get state() {
        if (this._store) {
            return this._store.get();
        }
        return [];
    }
    log() {
        console.log(this.state);
    }
    get IFluidHTMLView() { return this; }
    initializingFirstTime() {
        return __awaiter(this, void 0, void 0, function* () {
            const newState = SharedJson1.create(this.runtime);
            // const initialData = JSON.parse(JSON.stringify(myArray));
            newState.replace([], true, []);
            this.root.set("json1", newState.handle);
            const map = SharedMap.create(this.runtime);
            this.root.set("map", map.handle);
        });
    }
    hasInitialized() {
        var _a, _b, _c, _d;
        return __awaiter(this, void 0, void 0, function* () {
            this._store = yield ((_a = this.root.get("json1")) === null || _a === void 0 ? void 0 : _a.get());
            (_b = this._store) === null || _b === void 0 ? void 0 : _b.on("op", () => {
                this.render();
                this.emit("arrayModified");
            });
            this.log();
            this._map = yield ((_c = this.root.get("map")) === null || _c === void 0 ? void 0 : _c.get());
            console.log((_d = this._map) === null || _d === void 0 ? void 0 : _d.handle);
            this.root.on("valueChanged", (changed) => {
                this.render();
            });
        });
    }
    render(elm, options) {
        if (!this._elm) {
            // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
            this._elm = elm;
        }
        if (!elm) {
            // eslint-disable-next-line no-param-reassign
            elm = this._elm;
        }
        this.log();
        ReactDOM.render(React.createElement(React.Fragment, null,
            React.createElement(UI, { dataObject: this })), elm);
    }
    // IArray implementation
    insert(index, content) {
        var _a;
        (_a = this._store) === null || _a === void 0 ? void 0 : _a.insert([index], content);
        return Array.from(this.state);
    }
    push(content) {
        var _a;
        (_a = this._store) === null || _a === void 0 ? void 0 : _a.insert([this.length], content);
        return this.length;
    }
    unshift(content) {
        var _a;
        (_a = this._store) === null || _a === void 0 ? void 0 : _a.insert([0], content);
        return this.length;
    }
    delete(index, length) {
        var _a;
        (_a = this._store) === null || _a === void 0 ? void 0 : _a.remove([index]);
        return Array.from(this.state);
    }
    get(index) {
        return this.state[index];
    }
    set(index, content) {
        var _a;
        (_a = this._store) === null || _a === void 0 ? void 0 : _a.replace([index], true, content);
        return this.length;
    }
    clear() {
        var _a;
        (_a = this._store) === null || _a === void 0 ? void 0 : _a.replace([], true, []);
    }
    get length() {
        return this.state.length;
    }
}
const randomInt = (min, max) => {
    // eslint-disable-next-line no-param-reassign
    min = Math.ceil(min);
    // eslint-disable-next-line no-param-reassign
    max = Math.floor(max);
    return Math.floor(Math.random() * (max - min + 1)) + min;
};
export const UI = (props) => {
    const push = (dataobj) => {
        dataobj.push(dataobj.length);
        dataobj.log();
    };
    const unshift = (dataobj) => {
        dataobj.unshift(-dataobj.length);
        dataobj.log();
    };
    const remove = (dataobj) => {
        dataobj.delete(randomInt(0, dataobj.length - 1));
        dataobj.log();
    };
    const insert = (dataobj) => {
        dataobj.insert(randomInt(0, dataobj.length - 1), "inserted");
        dataobj.log();
    };
    const onAdd = (add) => {
        console.log(`onAdd`);
        const index = add.existing_value.length;
        props.dataObject.insert(index, add.new_value[index]);
        console.log(add);
    };
    const onDelete = (add) => {
        console.log(`onDelete`);
        if (add.name) {
            props.dataObject.delete(parseInt(add.name, 10));
        }
        console.log(add);
    };
    const onEdit = (add) => {
        console.log(`onEdit`);
        if (add.name) {
            props.dataObject.set(parseInt(add.name, 10), add.new_value);
        }
        console.log(add);
    };
    return (React.createElement("div", null,
        React.createElement("button", { onClick: (event) => push(props.dataObject) }, "Push"),
        React.createElement("button", { onClick: (event) => unshift(props.dataObject) }, "Unshift"),
        React.createElement("button", { onClick: (event) => remove(props.dataObject) }, "Delete"),
        React.createElement("button", { onClick: (event) => insert(props.dataObject) }, "Insert"),
        React.createElement("button", { onClick: (event) => props.dataObject.clear() }, "Clear"),
        React.createElement("button", { onClick: (event) => props.dataObject.log() }, "Log"),
        React.createElement(ReactJson, { src: props.dataObject.state, theme: "flat", enableClipboard: false, displayObjectSize: true, onAdd: onAdd, onDelete: onDelete, onEdit: onEdit })));
};
//# sourceMappingURL=ArrayDataObject.js.map